import javax.microedition.lcdui.*;
import java.util.Random;

public class WordixCanvas extends Canvas implements CommandListener {
    Wordix game;
    byte[][] matrix;

    int width, height;
    int delay;
    int x;
    int y;
    int dx;
    int dy;
    int x0;
    int y0;
    int stepx;
    int stepy;
    int rows;
    int columns;
    int result;
    char    currChar;
    byte    currByte;
    boolean running;
    Random  rand;
    Font    font;
    boolean repaintAll;
    boolean suspended;
    Display display;

    static final int WHITE = 0xffffff;
    static final int BLACK = 0;
    static final int MAX_DELAY = 1000;

    static char convertChar(byte b) { 
        return (char)(b < 0 ? b == -88 ? 0x401 : b == -72 ? 0x451 : 
                      (0x400 | ((b & 0x7f) - 0x30)) : b);
    }

    private void next() { 
        int direction = rand.nextInt() & 3;
        currByte = game.nextLetter();
        currChar = convertChar(currByte);
        x = x0;
        y = y0;
        result += 1;
        switch (direction) { 
        case 0:
            dx = 1;
            dy = 0;
            break;
        case 1:
            dx = -1;
            dy = 0;
            break;
        case 2:
            dx = 0;
            dy = 1;
            break;
        default:
            dx = 0;
            dy = -1;
            break;
        }
    }

    public WordixCanvas(Wordix game) {
        this.game = game;
        delay = MAX_DELAY - game.settings.level*MAX_DELAY/Settings.LEVELS;
	width = getWidth();
	height = getHeight();
        rand = new Random();
        font = Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_PLAIN, Font.SIZE_SMALL);
        stepy = font.getHeight();
        stepx = font.charWidth('w');
        rows = height / stepy;
        columns = width / stepx;
        x0 = columns >>> 1;
        y0 = rows >>> 1;
        matrix = new byte[columns][rows];
        next();
        addCommand(Wordix.STOP_CMD);
        addCommand(Wordix.BACK_CMD);
        repaintAll = true;
        display = Display.getDisplay(game);
    }

    void start() {         
        running = true;
        new TickThread().start();        
        setCommandListener(this);
        display.setCurrent(this);
    }
    
    void stop() { 
        running = false;
    }

    boolean tick() { 
        if (running) { 
            if (!suspended) { 
                repaint();
            }
            return true;
        }
        return false;
    }

    class TickThread extends Thread { 
        public void run() { 
            try { 
                while (tick()) {
                    sleep(delay);
                }
            } catch (InterruptedException x) {}
        }
    }           

    private void findWord(Graphics g) {
        int i = x;
        int j = y;
        while (i >= 0 && matrix[i][j] != 0) { 
            int len = game.getRowWordLength(matrix, i, j); 
            if (len > 0) { 
                g.setColor(WHITE);
                g.fillRect(i*stepx, j*stepy, stepx*len, stepy); 
                while (--len >= 0) { 
                    matrix[i++][j] = 0;
                }
                return;
            }
            i -= 1;
        }
        i = x;
        while (j >= 0 && matrix[i][j] != 0) { 
            int len = game.getColumnWordLength(matrix, i, j); 
            if (len > 0) { 
                g.setColor(WHITE);
                g.fillRect(i*stepx, j*stepy, stepx, stepy*len); 
                while (--len >= 0) { 
                    matrix[i][j++] = 0;
                }
                return;
            }
            j -= 1;
        }
    }

    protected void paint(Graphics g) {
        g.setFont(font);
        if (repaintAll) { 
            g.setColor(WHITE);
            g.fillRect(0, 0, width, height);
            repaintAll = false;
        }
        if (x + dx < 0 || y + dy < 0 || x + dx >= columns || y + dy >= rows
            || matrix[x+dx][y+dy] != 0) 
        {
            matrix[x][y] = currByte;
            findWord(g);
            if (matrix[x0][y0] != 0) { 
                gameOver();
                return;
            } else { 
                next();
            }
        } else { 
            g.setColor(WHITE);
            g.fillRect(x*stepx, y*stepy, stepx, stepy);
            x += dx;
            y += dy;
        }
        g.setColor(BLACK);
        g.drawChar(currChar, x*stepx, y*stepy, Graphics.TOP|Graphics.LEFT);
    }

    public void keyPressed(int keyCode) {
        switch (getGameAction(keyCode)) { 
        case LEFT:
            dx = -1;
            dy = 0;
            break;
        case RIGHT:
            dx = 1;
            dy = 0;
            break;
        case UP:
            dx = 0;
            dy = -1;
            break;
        case DOWN:
            dx = 0;
            dy = 1;
        }
    }

    void gameOver() { 
        String title = "Game over";
        running = false;
        if (result > game.settings.result) { 
            game.settings.result = result;
            game.saveSettings();
            title = "New record";
        }
        Alert a = new Alert(title, "Your result: " + result, null, AlertType.INFO);
        a.setTimeout(Alert.FOREVER);
        display.setCurrent(a, game.main);
    }        

    public void commandAction(Command c, Displayable d)  
    {
        if (c == Wordix.STOP_CMD) {
            suspended = true;
            removeCommand(Wordix.STOP_CMD);
            addCommand(Wordix.CONTINUE_CMD);
            suspended = true;
        } else if (c == Wordix.CONTINUE_CMD) { 
            suspended = false;
            removeCommand(Wordix.CONTINUE_CMD);
            addCommand(Wordix.STOP_CMD);            
        } else if (c == Wordix.BACK_CMD) {
            running = false;
            display.setCurrent(game.main);
        }
    }
}    
